@@ -56,6 +56,13 @@ static GFile *current_location = NULL;
 #define BUSY_IMAGE_PATH		BAOBAB_PIX_DIR "busy.gif"
 #define DONE_IMAGE_PATH		BAOBAB_PIX_DIR "done.png"
 
+enum {
+	DND_TARGET_URI_LIST
+};
+
+static GtkTargetEntry dnd_target_list[] = {
+	{ "text/uri-list", 0, DND_TARGET_URI_LIST },
+};
 
 static gboolean
 scan_is_local (GFile	*file)
@@ -87,10 +94,8 @@ baobab_set_busy (gboolean busy)
 		gedit_spinner_start (GEDIT_SPINNER (baobab.spinner));
 
 		baobab_chart_freeze_updates (baobab.rings_chart);
-		baobab_chart_set_summary_mode (baobab.rings_chart, FALSE);
 
 		baobab_chart_freeze_updates (baobab.treemap_chart);
-		baobab_chart_set_summary_mode (baobab.treemap_chart, FALSE);
 
 		gtk_widget_set_sensitive (baobab.chart_type_combo, FALSE);
 	}
@@ -109,6 +114,19 @@ baobab_set_busy (gboolean busy)
 	}
 }
 
+static void
+set_drop_target (GtkWidget *target, gboolean active) {
+	if (active) {
+		gtk_drag_dest_set (GTK_WIDGET (target),
+				   GTK_DEST_DEFAULT_DROP | GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT,
+				   dnd_target_list,
+				   G_N_ELEMENTS (dnd_target_list),
+				   GDK_ACTION_COPY);
+	} else {
+		gtk_drag_dest_unset (target);
+	}
+}
+
 /* menu & toolbar sensitivity */
 static void
 check_menu_sens (gboolean scanning)
@@ -142,6 +160,13 @@ check_menu_sens (gboolean scanning)
 	set_ui_widget_sens ("tb_scan_remote", !scanning);
 }
 
+static void
+check_drop_targets (gboolean scanning)
+{
+	set_drop_target (baobab.rings_chart, !scanning);
+	set_drop_target (baobab.treemap_chart, !scanning);
+}
+
 void
 baobab_scan_location (GFile *file)
 {
@@ -161,6 +186,7 @@ baobab_scan_location (GFile *file)
 	baobab.STOP_SCANNING = FALSE;
 	baobab_set_busy (TRUE);
 	check_menu_sens (TRUE);
+	check_drop_targets (TRUE);
 	gtk_tree_store_clear (baobab.model);
 	currentdepth = -1;	/* flag */
 	iterstack = g_queue_new ();
@@ -190,6 +216,7 @@ baobab_scan_location (GFile *file)
 
 	baobab_set_busy (FALSE);
 	check_menu_sens (FALSE);
+	check_drop_targets (FALSE);
 	set_statusbar (_("Ready"));
 
 	gtk_tree_view_columns_autosize (GTK_TREE_VIEW (baobab.tree_view));
@@ -325,8 +352,27 @@ first_row (void)
 	gdouble perc;
 	char *label;
 
+        GtkTreeIter root_iter;
+
+	gchar *capacity_label, *capacity_size;
+
+	gtk_tree_store_append (baobab.model, &root_iter, NULL);
+	capacity_size = g_format_size_for_display (g_fs.total);
+
+	capacity_label = g_strdup (_("Total filesystem capacity"));
+	gtk_tree_store_set (baobab.model, &root_iter,
+			    COL_DIR_NAME, capacity_label,
+			    COL_H_PARSENAME, "",
+			    COL_H_PERC, 100.0,
+			    COL_DIR_SIZE, capacity_size,
+			    COL_H_SIZE, g_fs.total,
+			    COL_H_ALLOCSIZE, g_fs.total,
+			    COL_H_ELEMENTS, -1, -1);
+	g_free (capacity_label);
+	g_free (capacity_size);
+
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (baobab.tree_view), FALSE);
-	gtk_tree_store_append (baobab.model, &firstiter, NULL);
+	gtk_tree_store_append (baobab.model, &firstiter, &root_iter);
 	size = g_format_size_for_display (g_fs.used);
 	if (g_fs.total == 0 && g_fs.used == 0) {
 		perc = 100.0;
@@ -335,7 +381,7 @@ first_row (void)
 		perc = ((gdouble) g_fs.used * 100) / (gdouble) g_fs.total;
 	}
 
-	label = g_strdup_printf ("<i>%s</i>", _("Total filesystem usage:"));
+	label = g_strdup (_("Total filesystem usage"));
 	gtk_tree_store_set (baobab.model, &firstiter,
 			    COL_DIR_NAME, label,
 			    COL_H_PARSENAME, "",
@@ -347,6 +393,8 @@ first_row (void)
 
 	g_free (size);
 	g_free (label);
+
+	gtk_tree_view_expand_all (GTK_TREE_VIEW (baobab.tree_view));
 }
 
 /* fills model during scanning */
@@ -884,8 +932,46 @@ create_context_menu (void)
 	gtk_widget_show_all (menu->widget);
 }
 
+
 static void
-initialize_ringschart (void)
+drag_data_received_handl (GtkWidget *widget,
+			 GdkDragContext *context,
+			 gint x,
+			 gint y,
+			 GtkSelectionData *selection_data,
+			 guint target_type,
+			 guint time,
+			 gpointer data)
+{
+	GFile *gf = NULL;
+	
+	/* set "gf" if we got some valid data */
+	if ((selection_data != NULL) &&
+	    (selection_data->length >= 0) &&
+	    (target_type == DND_TARGET_URI_LIST)) {
+		gchar **uri_list;
+		uri_list = g_uri_list_extract_uris ((gchar*)selection_data->data);
+		/* check list is 1 item long */
+		if (uri_list != NULL && uri_list[0] != NULL && uri_list[1] == NULL) {
+			gf = g_file_new_for_uri (uri_list[0]);
+		}
+		g_strfreev (uri_list);
+	}
+	
+	/* success if "gf" has been set */
+	if (gf != NULL) {
+		/* finish drop before beginning scan, as the drag-drop can
+		   probably time out */
+		gtk_drag_finish (context, TRUE, FALSE, time);
+		baobab_scan_location (gf);
+		g_object_unref (gf);
+	} else {
+		gtk_drag_finish (context, FALSE, FALSE, time);
+	}
+}
+
+static void
+initialize_charts (void)
 {
 	GtkWidget *hpaned_main;
 	GtkWidget *chart_frame;
@@ -935,6 +1021,8 @@ initialize_ringschart (void)
 					G_CALLBACK (on_chart_item_activated), NULL);
 	g_signal_connect (baobab.treemap_chart, "button-release-event", 
 					G_CALLBACK (on_chart_button_release), NULL);
+	g_signal_connect (baobab.treemap_chart, "drag-data-received",
+					G_CALLBACK (drag_data_received_handl), NULL);
 	gtk_widget_show (baobab.treemap_chart);
 	/* Ends Baobab's Treemap Chart */
 
@@ -957,6 +1045,8 @@ initialize_ringschart (void)
 					G_CALLBACK (on_chart_item_activated), NULL);
 	g_signal_connect (baobab.rings_chart, "button-release-event", 
 					G_CALLBACK (on_chart_button_release), NULL);
+	g_signal_connect (baobab.rings_chart, "drag-data-received",
+					G_CALLBACK (drag_data_received_handl), NULL);
 	gtk_widget_show (baobab.rings_chart);
 	/* Ends Baobab's Treemap Chart */
 
@@ -970,6 +1060,8 @@ initialize_ringschart (void)
 	gtk_widget_show_all (chart_frame);
 
 	create_context_menu ();
+
+	check_drop_targets (FALSE);
 }
 
 static gboolean
@@ -1077,7 +1169,7 @@ main (int argc, char *argv[])
 	set_statusbar (_("Ready"));
 
 	/* The ringschart */
-	initialize_ringschart ();
+	initialize_charts ();
 
 	/* commandline */
 	if (directories && directories[0]) {
